﻿//******************************************************************************
//Name          : Leakage.cs
//Description   : Implementation to experience the beauty and the pitfalls of Leakage
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;
using Display;


class Spectrum
{
    const int samplerate = 128;                         // Abtastwerte im Abtastintervall
    const double PI      =   3.141592653589793;

    public double[] signal1 = new double[samplerate];   // Signalverlauf
    public double[] signal2 = new double[samplerate];   // Signalverlauf
    public double[] arg = new double[samplerate];
    public double[] AS = new double[samplerate];        // Amplitudenspektrum
    public double[] PS = new double[samplerate];        // Phasenspektrum

    public void CreateSignals()
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal1[k] = Math.Sin(2 * PI * 12 * k / samplerate);
            signal2[k] = Math.Sin(2 * PI * 12 * 0.95 * k / samplerate);
        }

        // store argument data
        for (int k = 0; k < samplerate; k++)
        {
            arg[k] = k;// 2 * PI * k / samplerate;
        }
    }

    public void Hanning(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (0.5 - 0.5 * Math.Cos(2 * PI * k / samplerate));
        }
    }

    public void Hamming(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (0.54 - 0.46 * Math.Cos(2 * PI * k / samplerate));
        }
    }

    public void Blackman(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (0.42 - 0.5 * Math.Cos(2 * PI * k / samplerate) + 0.08 * Math.Cos(4 * PI * k / samplerate));
        }
    }

    public void BlackmanExact(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (0.42659071 - 0.49656062 * Math.Cos(2 * PI * k / samplerate) + 0.07684867 * Math.Cos(4 * PI * k / samplerate));
        }
    }

    public void BlackmanHarris(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (0.42323 - 0.49755 * Math.Cos(2 * PI * k / samplerate) + 0.07922 * Math.Cos(4 * PI * k / samplerate));
        }
    }

    public void FlatTop(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (0.2810639 - 0.5208972 * Math.Cos(2 * PI * k / samplerate) + 0.1980399 * Math.Cos(4 * PI * k / samplerate));
        }
    }

    public void Bartlett(ref double[] signal)
    {
        for (int k = 0; k < samplerate; k++)
        {
            signal[k] *= (1 - Math.Abs((double)(k - (samplerate / 2)) / (samplerate / 2)));
        }
    }

    public void DFT(double[] signal)
    {
        double real = 0;
        double imag = 0;

        for (int freq = 0; freq < samplerate; freq++)
        {
            real = 0;
            imag = 0;

            for (int k = 0; k < samplerate; k++)
            {
                real += signal[k] * Math.Cos(2*PI*freq*k/samplerate); // an
                imag += signal[k] * Math.Sin(2*PI*freq*k/samplerate); // bn
            }
            real *= 2.0 / samplerate;
            imag *= 2.0 / samplerate;

            AS[freq] = Math.Sqrt(real*real + imag*imag);
            PS[freq] = Math.Atan2(imag, real + 1E-10);
        }
        AS[0] /= 2.0;   // Korrektur Gleichanteil
    }
}


class MainApp
{
    [STAThread]
    static void Main()
    {
        Spectrum mySpectrumObj = new Spectrum();

        mySpectrumObj.CreateSignals();

        ///////////////////////////////////////////
        // compute and display DFT of 1st signal //
        ///////////////////////////////////////////
        mySpectrumObj.DFT(mySpectrumObj.signal1);
        // create a chart for displaying that data...
        DisplayChart mydisplay1 = new DisplayChart(720, 480, SeriesChartType.Line, "Time domain", "k", "x(k)");
        mydisplay1.AddData(mySpectrumObj.arg, mySpectrumObj.signal1);

        DisplayChart mydisplay2 = new DisplayChart(720, 480, SeriesChartType.Column, "Frequency domain", "f", "|X(f)|");
        mydisplay2.AddData(mySpectrumObj.arg, mySpectrumObj.AS);

        ///////////////////////////////////////////
        // compute and display DFT of 2nd signal //
        ///////////////////////////////////////////
        mySpectrumObj.DFT(mySpectrumObj.signal2);
        // create a chart for displaying that data...
        DisplayChart mydisplay3 = new DisplayChart(720, 480, SeriesChartType.Line, "Time domain", "k", "x(k)");
        mydisplay3.AddData(mySpectrumObj.arg, mySpectrumObj.signal2);

        DisplayChart mydisplay4 = new DisplayChart(720, 480, SeriesChartType.Column, "Frequency domain", "f", "|X(f)|");
        mydisplay4.AddData(mySpectrumObj.arg, mySpectrumObj.AS);

        ///////////////////////////////////////////////////////////////////
        // compute and display DFT of 2nd signal after Hanning-windowing //
        ///////////////////////////////////////////////////////////////////
        mySpectrumObj.Hanning(ref mySpectrumObj.signal2);
        mySpectrumObj.DFT(mySpectrumObj.signal2);
        // create a chart for displaying that data...
        DisplayChart mydisplay5 = new DisplayChart(720, 480, SeriesChartType.Line, "Time domain", "k", "x(k)");
        mydisplay5.AddData(mySpectrumObj.arg, mySpectrumObj.signal2);

        DisplayChart mydisplay6 = new DisplayChart(720, 480, SeriesChartType.Column, "Frequency domain", "f", "|X(f)|");
        mydisplay6.AddData(mySpectrumObj.arg, mySpectrumObj.AS);

        // ...and run the chart-application
        Application.EnableVisualStyles();
        Application.Run(mydisplay1);
        Application.Run(mydisplay2);
        Application.Run(mydisplay3);
        Application.Run(mydisplay4);
        Application.Run(mydisplay5);
        Application.Run(mydisplay6);

        mydisplay1.Dispose();
        mydisplay2.Dispose();
        mydisplay3.Dispose();
        mydisplay4.Dispose();
        mydisplay5.Dispose();
        mydisplay6.Dispose();
        Application.Exit();
    }
}
